@@ -52,6 +52,7 @@ public:
       bool trim_ws,
       const size_t skip,
       const char* comment,
+      const bool skip_empty_rows,
       const size_t n_max,
       const bool progress)
       : col_starts_(col_starts),
@@ -77,10 +78,12 @@ public:
 
     size_t file_size = mmap_.size();
 
-    size_t start = find_first_line(mmap_, skip, comment);
+    size_t start = find_first_line(
+        mmap_, skip, comment, skip_empty_rows, /* embedded_nl */ false);
 
     // Check for windows newlines
-    size_t first_nl = find_next_newline(mmap_, start, false);
+    size_t first_nl = find_next_newline(
+        mmap_, start, comment, skip_empty_rows, /* embedded_nl */ false);
     windows_newlines_ = first_nl > 0 && mmap_[first_nl - 1] == '\r';
 
     std::unique_ptr<RProgress::RProgress> pb = nullptr;
@@ -94,14 +97,27 @@ public:
 #endif
     }
 
+    bool n_max_set = n_max != static_cast<size_t>(-1);
+
     if (n_max > 0) {
       newlines_.push_back(start - 1);
     }
 
     index_region(
-        mmap_, newlines_, start, file_size - 1, 0, n_max, pb, file_size / 1000);
+        mmap_,
+        newlines_,
+        start,
+        file_size - 1,
+        0,
+        comment,
+        skip_empty_rows,
+        n_max,
+        pb,
+        file_size / 1000);
 
-    newlines_.push_back(file_size - 1);
+    if (!n_max_set) {
+      newlines_.push_back(file_size - 1);
+    }
 
     if (progress) {
 #ifndef VROOM_STANDALONE
@@ -110,27 +126,26 @@ public:
     }
 
 #ifdef VROOM_LOG
-#if SPDLOG_ACTIVE_LEVEL <= SPD_LOG_LEVEL_DEBUG
     auto log = spdlog::basic_logger_mt(
         "basic_logger", "logs/fixed_width_index.idx", true);
+    log->set_level(spdlog::level::debug);
     for (auto& v : newlines_) {
       SPDLOG_LOGGER_DEBUG(log, "{}", v);
     }
     SPDLOG_LOGGER_DEBUG(log, "end of idx {0:x}", (size_t)&newlines_);
     spdlog::drop("basic_logger");
-#endif
 #endif
   }
 
-  size_t num_rows() const { return newlines_.size() - 1; }
-  size_t num_columns() const { return col_starts_.size(); }
+  size_t num_rows() const override { return newlines_.size() - 1; }
+  size_t num_columns() const override { return col_starts_.size(); }
 
-  std::string get_delim() const {
+  std::string get_delim() const override {
     /* TODO: FIXME */
     return "";
   }
 
-  string get(size_t row, size_t col) const {
+  string get(size_t row, size_t col) const override {
     auto begin = mmap_.data() + (newlines_[row] + 1 + col_starts_[col]);
     auto line_end = mmap_.data() + (newlines_[row + 1]) - windows_newlines_;
     const char* end;
@@ -159,27 +174,30 @@ public:
   public:
     column_iterator(std::shared_ptr<const fixed_width_index> idx, size_t column)
         : idx_(idx), column_(column), i_(0) {}
-    void next() { ++i_; }
-    void prev() { --i_; }
-    void advance(ptrdiff_t n) { i_ += n; }
-    bool equal_to(const base_iterator& it) const {
+    void next() override { ++i_; }
+    void prev() override { --i_; }
+    void advance(ptrdiff_t n) override { i_ += n; }
+    bool equal_to(const base_iterator& it) const override {
       return i_ == static_cast<const column_iterator*>(&it)->i_;
     }
-    ptrdiff_t distance_to(const base_iterator& it) const {
+    ptrdiff_t distance_to(const base_iterator& it) const override {
       return static_cast<ptrdiff_t>(
                  static_cast<const column_iterator*>(&it)->i_) -
              static_cast<ptrdiff_t>(i_);
     }
-    string value() const { return idx_->get(i_, column_); }
-    column_iterator* clone() const { return new column_iterator(*this); }
-    string at(ptrdiff_t n) const { return idx_->get(n, column_); }
-    std::string filename() const { return idx_->filename_; }
-    size_t index() const { return i_ / idx_->num_columns(); }
-    size_t position() const { return i_; }
+    string value() const override { return idx_->get(i_, column_); }
+    column_iterator* clone() const override {
+      return new column_iterator(*this);
+    }
+    string at(ptrdiff_t n) const override { return idx_->get(n, column_); }
+    std::string filename() const override { return idx_->filename_; }
+    size_t index() const override { return i_ / idx_->num_columns(); }
+    size_t position() const override { return i_; }
     virtual ~column_iterator() = default;
   };
 
-  std::shared_ptr<vroom::index::column> get_column(size_t column) const {
+  std::shared_ptr<vroom::index::column>
+  get_column(size_t column) const override {
     auto begin = new column_iterator(shared_from_this(), column);
     auto end = new column_iterator(shared_from_this(), column);
     end->advance(num_rows());
@@ -194,23 +212,26 @@ public:
       size_t start,
       size_t end,
       size_t offset,
+      const char* comment,
+      const bool skip_empty_rows,
       size_t n_max,
       std::unique_ptr<RProgress::RProgress>& pb,
       size_t update_size = -1) {
 
-    size_t pos = find_next_newline(source, start, false);
+    size_t pos = find_next_newline(
+        source, start, comment, skip_empty_rows, /* embededd_nl */ false);
 
     size_t lines_read = 0;
     auto last_tick = start;
 
     while (pos < end) {
       ++lines_read;
+      destination.push_back(offset + pos);
+
       if (lines_read >= n_max) {
         return lines_read;
       }
 
-      destination.push_back(offset + pos);
-
       if (pb) {
         auto tick_size = pos - last_tick;
         if (tick_size > update_size) {
@@ -219,7 +240,8 @@ public:
         }
       }
 
-      pos = find_next_newline(source, pos + 1, false);
+      pos = find_next_newline(
+          source, pos + 1, comment, skip_empty_rows, /* embedded_nl */ false);
     }
 
     if (pb) {
@@ -229,11 +251,11 @@ public:
     return lines_read;
   }
 
-  std::shared_ptr<vroom::index::row> get_row(size_t) const {
+  std::shared_ptr<vroom::index::row> get_row(size_t) const override {
     // TODO: UNUSED
     return nullptr;
   }
-  std::shared_ptr<vroom::index::row> get_header() const {
+  std::shared_ptr<vroom::index::row> get_header() const override {
     // TODO: UNUSED
     return nullptr;
   }
