@@ -1,8 +1,11 @@
 #include "vroom_time.h"
 
 double parse_time(
-    const string& str, DateTimeParser& parser, const std::string& format) {
-  parser.setDate(str.begin(), str.end());
+    const char* begin,
+    const char* end,
+    DateTimeParser& parser,
+    const std::string& format) {
+  parser.setDate(begin, end);
   bool res = (format == "") ? parser.parseLocaleTime() : parser.parse(format);
 
   if (res) {
@@ -19,19 +22,33 @@ cpp11::doubles read_time(vroom_vec_info* info) {
 
   cpp11::writable::doubles out(n);
 
+  auto err_msg = info->format.size() == 0
+                     ? std::string("time in ISO8601")
+                     : std::string("time like ") + info->format;
+
   parallel_for(
       n,
       [&](size_t start, size_t end, size_t) {
         R_xlen_t i = start;
         DateTimeParser parser(info->locale.get());
         auto col = info->column->slice(start, end);
-        for (const auto& str : *col) {
-          out[i++] = parse_time(str, parser, info->format);
+        for (auto b = col->begin(), e = col->end(); b != e; ++b) {
+          out[i++] = parse_value<double>(
+              b,
+              col,
+              [&](const char* begin, const char* end) -> double {
+                return parse_time(begin, end, parser, info->format);
+              },
+              info->errors,
+              err_msg.c_str(),
+              *info->na);
         }
       },
       info->num_threads,
       true);
 
+  info->errors->warn_for_errors();
+
   out.attr("class") = {"hms", "difftime"};
   out.attr("units") = "secs";
 
